unit fAuthors;
{

This is a simple Example showing how to create a descendant XMLCollection and
XMLCollectionItem. This includes samples for most of the data types supported.

The "Generate" button will populate the top memo with XML generated from a
collection with 3 items.  Clicking "Save" will save a "test.xml" file to the
current directory that can be opened in your favorite XML editor, or IE5+.

"Parse" will read the xml from the top memo and populate a collection.  It then
demonstrates how easy it is to access this data, printing out the last name of
each author from the list.
}

interface

uses
  Classes, Controls, Dialogs, Forms, Graphics, Messages, SysUtils, StdCtrls, Windows
  , XMLWorks2;

type
  TfrmAuthors = class(TForm)
    memoTestXML: TMemo;
    memoTestParse: TMemo;
    btnGenerate: TButton;
    btnParse: TButton;
    lblGenerate: TStaticText;
    lblParse: TStaticText;
    btnSave: TButton;
    procedure btnGenerateClick(Sender: TObject);
    procedure btnParseClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


  // Testing Classes

  TAuthorType = (atWorthless, atGood, atBad);

  TAuthorCollection = class(TXMLCollection)
  end;

  TAuthorCollectionItem = class(TXMLCollectionItem)
  private
    fphone: String;
    fau_id: String;
    faddress: String;
    fcity: String;
    fpostalcode: String;
    fau_lname: String;
    fstate: String;
    fau_fname: String;
    ftestint : Integer;
    ftestchar: char;
    ftestfloat: real;
    ftestint64: Int64;
    ftestenum: TAuthorType;
    ftestvariant: Variant;

  public
  published
    property au_id : String read fau_id write fau_id;
    property au_lname : String read fau_lname write fau_lname;
    property au_fname : String read fau_fname write fau_fname;
    property phone : String read fphone write fphone;
    property address : String read faddress write faddress;
    property city : String read fcity write fcity;
    property state : String read fstate write fstate;
    property postalcode : String read fpostalcode write fpostalcode;

    property TestInt : Integer read ftestint write ftestint;
    property TestFloat : real read ftestfloat write ftestfloat;
    property TestVariant : Variant read ftestvariant write ftestvariant;
    property TestInt64 : Int64 read ftestint64 write ftestint64;
    property TestChar : char read ftestchar write ftestchar;
    property TestEnum : TAuthorType read ftestenum write ftestEnum;
  end;

var
  frmAuthors: TfrmAuthors;

implementation

{$R *.DFM}

procedure TfrmAuthors.btnGenerateClick(Sender: TObject);

begin
  btnGenerate.Enabled := False;
  with TAuthorCollection.Create(TAuthorCollectionItem) do
  try
    // Here we are going to add three new authors to the Collection
    with TAuthorCollectionItem(Add) do
    begin
      au_fname := 'Marc';
      au_lname := 'Bir';
      phone := '713-555-8931';
      address := '1234 Somestreet';
      city := 'Somecity';
      state := 'TX';
      postalcode := '12345';

      TestInt := 20;
      TestFloat := 1.1;
      TestVariant := 'TestVariant';
      TestInt64 := 205876666;
      TestChar := 'Z';
      TestEnum := atWorthless;
    end;
    with TAuthorCollectionItem(Add) do
    begin
      au_fname := 'Sancho';
      au_lname := 'Fock';
      phone := '01-111-555-5555';
      address := '1234 Somestreet';
      city := 'Somecity';
      state := 'DE';
      postalcode := '12345';

      TestInt := 99;
      TestFloat := 6.1;
      TestVariant := true;
      TestInt64 := 203333333;
      TestChar := 'A';
      TestEnum := atGood;
    end;
    with TAuthorCollectionItem(Add) do
    begin
      au_fname := 'Thomas';
      au_lname := 'Weinert';
      phone := '02-222-555-5555';
      address := '8888 Somestreet';
      city := 'Somecity';
      state := 'DE';
      postalcode := '54321';

      TestInt := 1;
      TestFloat := Pi;
      TestVariant := False;
      TestInt64 := 99;
      TestChar := 'X';
      TestEnum := atBad;
    end;
    memoTestXML.Lines.Clear;


    // To generate the XML, simply read the "XML" property of the XMLObject
    memoTestXML.Lines.Add(XML);
  finally
    Free;
    btnGenerate.Enabled := True;
  end;
end;

procedure TfrmAuthors.btnParseClick(Sender: TObject);
var
  I: Integer;
begin
  btnParse.Enabled := False;
  memoTestParse.Lines.Clear;

  with TAuthorCollection.Create(TAuthorCollectionItem) do
  try
    XML := memoTestXML.Lines.Text;
    for I := 0 to Count - 1 do    // Iterate
      memoTestParse.Lines.Add(TAuthorCollectionItem(Items[I]).fau_lname);
  finally
    Free;
    btnParse.Enabled := True;
  end;

end;

procedure TfrmAuthors.btnSaveClick(Sender: TObject);
begin
  memoTestXML.Lines.SaveToFile('Test.xml');
end;

end.
