unit FastStringCases;

interface
uses
  SysUtils, FastStrings, FastStringFuncs, TestFrameWork;

type
  TFastStringsCase = class(TTestCase)
  published
    procedure TestMakeBMTable;
    procedure TestMakeBMTableNoCase;
    procedure TestFastPos;
    procedure TestFastReplace;
    procedure TestStringCount;
  end;

implementation

{ TFastStringsCase }

procedure TFastStringsCase.TestFastPos;
var
  S: string;

  procedure Find(Value: string);
  begin
    Check(FastPos(S, Value, Length(S), Length(Value), 1) = Pos(Value, S),
      'Result of FastPos ' + Value + ' is incorrect');
    Check(FastPosNoCase(S, Value, Length(S), Length(Value), 1) = Pos(UpperCase(Value), UpperCase(S)),
      'Result of FastPosNoCase ' + Value + ' is incorrect');
  end;
begin
  S := 'the cat sat on the mat';
  Find('sat');
  Find('the');
  Find('cat');
  Find('on');
  Find('mat');
  Find('a');
  Find('z');
end;

procedure TFastStringsCase.TestFastReplace;
var
  S: string;
begin
  S := 'the cat sat on the mat';
  Check(
    FastReplace(S, 'cat', 'dog', True) =
    StringReplace(S, 'cat', 'dog', [rfReplaceAll]),
    'FastReplace test 1 not working');
  Check(FastReplace(S, 'Cat', 'dog', True) = S, 'Replaced when shouldn''t');
  Check(
    FastReplace(S, 'Cat', 'DOG', False) = 'the DOG sat on the mat',
    'Didn''t change Cat to DOG');
end;

procedure TFastStringsCase.TestMakeBMTable;
var
  I: Integer;
  Table: TBMJumpTable;
begin
  MakeBMTable('cat', 3, Table);
  for I := 0 to 255 do
    Check((Table[I] = 3) or (Table[I] = 3 - Pos(Chr(I), 'cat')),
      'Letter ' + Chr(I) + ' is wrong');
end;

procedure TFastStringsCase.TestMakeBMTableNoCase;
var
  I: Integer;
  Table: TBMJumpTable;
begin
  MakeBMTableNoCase('Cat', 3, Table);
  for I := 0 to 255 do
    Check((Table[I] = 3) or (Table[I] = 3 - Pos(UpCase(Chr(I)), 'CAT')),
      'Letter ' + Chr(I) + ' is wrong');
end;

procedure TFastStringsCase.TestStringCount;
var
  I: Integer;
  S: string;
begin
  S := '';
  for I := 1 to 5 do S := S + 'The cat sat on the mat';
  Check(StringCount(S, 'the', False) = 10, 'StringCount No Case 1');
  Check(StringCount(S, 'The', True) = 5, 'StringCount No Case 2');
  Check(StringCount(S, 'mat', True) = 5, 'StringCount No Case 3');
end;

initialization
 TestFramework.RegisterTest(TFastStringsCase.Suite);
end.

