unit VariantTestCases;

interface
(*
  VariantTestCases
  Version   : 1.0
  Date      : 5/5/2002

  Descripton:

  Author: Sancho Fock
          http://www.thsfock.de
          sancho@thsfock.de

  Description

    This Unit contains the Test Cases for the Variant to XML interface of
    the Unit XMLVariants
    *)

uses
  TestFramework, XMLWorks2;

type
 TXMLSimpleObject = class(TXMLInterfacedObject)
 private
   FValue : string;
 published
   property Value : string read FValue write FValue;
 end;

 TVariantTestCase = class(TTestCase)
 private
   FsrcObj      : TXMLSimpleObject;
   FdestObj     : TXMLSimpleObject;
   FdestObj2     : TXMLSimpleObject;
   FSourceIntf  : IXMLWorksObject;
   FDestIntf    : IXMLWorksObject;
   FDestIntf2    : IXMLWorksObject;
   FvarEmpty    : variant;
   FvarNull     : variant;
   FvarSmallint : variant;
   FvarInteger  : variant;
   FvarSingle   : variant;
   FvarDouble   : variant;
   FvarCurrency : variant;
   FvarDate     : variant;
//   FvarDispatch : variant;  { TODO : Testing a Dispached Iterface }
   FvarBoolean  : variant;
   FvarUnknown  : variant;
   FvarByte     : variant;
   FvarOleStr   : variant;
   FvarString   : variant;
   FSimpleArray : variant;
   FMultiDimArray : variant;
   function createVarArrayOfFloats : variant;
   function createVarArrayOfInteger : variant;
   function createVarArrayOfDate : variant;
   procedure compareSingleDimArray(newArray, oldArray : Variant);
   function createSimpleArray : variant;
   procedure compareSimpleArray(newArray : Variant);
   function createMultiDimArray : variant;
   procedure compareMultiDimArray(newArray : Variant);
 protected
   procedure SetUp ; override;
 published
   procedure TestVarEmptyWorks;
   procedure TestVarNullWorks;
   procedure TestVarSmallintWorks;
   procedure TestVarIntegerWorks;
   procedure TestVarSingleWorks;
   procedure TestVarDoubleWorks;
   procedure TestVarCurrencyWorks;
   procedure TestVarDateWorks;
   procedure TestVarBooleanWorks;
   procedure TestVarUnknownWorks;
   procedure TestVarByteWorks;
   procedure TestVarOLEStrWorks;
   procedure TestVarStringWorks;
   procedure TestSimpleVarArrayWorks;
   procedure TestSingleDimFloatVarArrayWorks;
   procedure TestSingleDimIntegerVarArrayWorks;
   procedure TestSingleDimDateVarArrayWorks;
   procedure TestMultidimensionalVarArrayWorks;
 end;

const
  SI_VALUE     = 5;
  INT_VALUE    = 12345678;
  SINGLE_VALUE = 0.4;
  DOUBLE_VALUE = 12345678;
  CURR_VALUE   = 7489.90;
  DATE_VALUE   = 5000.200;
  STR_VALUE    = 'A Test string Value';
  BOOL_VALUE   = True;
  BYTE_VALUE   = 3;

implementation

uses
  XMLVariants,
  {$IFDEF VER140} // New Variant Unit for Delphi 6
  Variants,
  {$ENDIF}
  sysutils;


function TVariantTestCase.createVarArrayOfFloats : variant;
begin
  result := VarArrayCreate([-1,3],varDouble);
  result[-1] := 0.23;
  result[0]  := 1;
  result[1]  := -2.3;
  result[2]  := 23.45678;
end;

function TVariantTestCase.createVarArrayOfInteger : variant;
begin
  result := VarArrayCreate([-1,3],varInteger);
  result[-1] := -1;
  result[0]  := 1;
  result[1]  := 0;
  result[2]  := 2345678;
end;

function TVariantTestCase.createVarArrayOfDate : variant;
begin
  result := VarArrayCreate([-1,3],varDate);
  result[-1] := date;
  result[0]  := date-100;
  result[1]  := date+100;
  result[2]  := 0;
end;

procedure TVariantTestCase.compareSingleDimArray(newArray, oldArray : Variant);
var
  i : Integer;
begin
  for i := -1 to 3 do begin
    Check(VarType(newArray[i])=VarType(oldArray[i]), 'state is changed at '+IntToStr(i)+'-- src is:'+VarTypeAsString(VarType(oldArray[i]))+'-- dst is: '+VarTypeAsString(VarType(oldArray[i])));
    Check(newArray[i] = oldArray[i], 'Value is lost at '+IntToStr(i));
  end
end;


function TVariantTestCase.createSimpleArray : variant;
begin
  result := VarArrayCreate([-1,3],varVariant);
  result[-1] := FvarSmallint;
  result[0]  := FvarString;
  result[1]  := FvarDate;
  result[2]  := FvarByte;
  result[3]  := FvarCurrency;
end;

procedure TVariantTestCase.compareSimpleArray(newArray : Variant);
var
  i : Integer;
begin
  for i := -1 to 3 do begin
    Check(VarType(newArray[i])=VarType(FSimpleArray[i]), 'state is changed at '+IntToStr(i)+'-- src is:'+VarTypeAsString(VarType(FSimpleArray[i]))+'-- dst is: '+VarTypeAsString(VarType(FSimpleArray[i])));
    Check(newArray[i] = FSimpleArray[i], 'Value is lost at '+IntToStr(i));
  end
end;

function TVariantTestCase.createMultiDimArray : variant;
begin
  result := VarArrayCreate([-1,0,0,1,1,4],varVariant);
  result[-1,0,1] := FvarEmpty;
  result[-1,0,2] := FvarNull;
  result[-1,0,3] := FvarBoolean;
  result[-1,0,4] := FvarBoolean;

  result[-1,1,1] := FvarSmallint;
  result[-1,1,2] := FvarInteger;
  result[-1,1,3] := FvarUnknown;
  result[-1,1,4] := FvarString;

  result[0,0,1] := FvarSingle;
  result[0,0,2] := FvarDouble;
  result[0,0,3] := FvarByte;
  result[0,0,4] := FvarByte;

  result[0,1,1] := FvarCurrency;
  result[0,1,2] := FvarDate;
  result[0,1,3] := FSimpleArray;
  result[0,1,4] := FvarOleStr;
end;

procedure TVariantTestCase.compareMultiDimArray(newArray : Variant);
var
  a,b,c : integer;
begin
  check(VarArrayDimCount(newArray) = 3,'new Array has not proper Dimensions');
  check(VarArrayDimCount(FMultiDimArray) = 3,'new Array has not proper Dimensions');
  for a := -1 to 0 do begin
    for b := 0 to 1 do begin
      for c := 1 to 4 do begin
        if (a=-1) and (b=1) and (c=3) then begin  //Interface
          Check((FsrcObj.Value = FdestObj2.Value), 'Property Value is lost');
        end else begin
          if (a=0) and (b=1) and (c=3) then begin // simpleArray
            self.compareSimpleArray(newArray[a,b,c]);
          end else begin
            Check(VarType(newArray[a,b,c])=VarType(FMultiDimArray[a,b,c])
                , 'state is changed at '+IntToStr(a)+','+IntToStr(b)+','+IntToStr(c)+'-- src is:'+VarTypeAsString(VarType(FMultiDimArray[a,b,c]))+'-- dst is: '+VarTypeAsString(VarType(FMultiDimArray[a,b,c])));
            if not (VarType(newArray[a,b,c]) = varEmpty) then
              Check(newArray[a,b,c] = FMultiDimArray[a,b,c], 'Value is lost at '+IntToStr(a)+','+IntToStr(b)+','+IntToStr(c));
          end;
        end;
      end;
    end;
  end;
end;

procedure TVariantTestCase.SetUp;
var
  doub : double;
  curr : currency;
  Date : TDateTime;
  sInt : SmallInt;
begin
  inherited setUp;
  doub := DOUBLE_VALUE;
  curr := CURR_VALUE;
  Date := DATE_VALUE;
  sInt := SI_VALUE;
  FsrcObj   := TXMLSimpleObject.create;
  FsrcObj.Value := STR_VALUE;
  FdestObj  := TXMLSimpleObject.create;
  FdestObj2 := TXMLSimpleObject.create;
  FSourceIntf  := FsrcObj;
  FDestIntf    := FdestObj;
  FDestIntf2   := FdestObj2;
  FvarNull     := null;
  FvarSmallint := sInt;
  FvarSmallint := VarAsType(FvarSmallint,varSmallint);
  FvarInteger  := VarAsType(INT_VALUE,varInteger);
  FvarSingle   := SINGLE_VALUE;
  FvarSingle   := VarAsType(FvarSingle,varSingle);
  FvarDouble   := Doub;
  FvarCurrency := Curr;
  FvarDate     := Date;
// FvarOleStr   := OleStrToStrVar(StringToOleStr(STR_VALUE));
  FvarBoolean  := BOOL_VALUE;
  FvarUnknown  := FSourceIntf;
  FvarByte     := Byte(BYTE_VALUE);
  FvarByte     := VarAsType(FvarByte,varByte);
  FvarOleStr   := WideString(STR_VALUE);
  FvarString   := STR_VALUE;
  FSimpleArray := self.createSimpleArray;
  FMultiDimArray := self.createMultiDimArray;
end;

procedure TVariantTestCase.TestVarEmptyWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarEmpty)=varEmpty, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarEmpty)));
  XMLToVariant(VariantToXML(FvarEmpty),newVariant);
  Check(VarType(newVariant)=varEmpty, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
end;

procedure TVariantTestCase.TestVarNullWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarNull)=varNull, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarNull)));
  XMLToVariant(VariantToXML(FvarNull),newVariant);
  Check(VarType(newVariant)=varNull, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = Null, 'Value is lost');
end;

procedure TVariantTestCase.TestVarSmallintWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarSmallint)=varSmallint, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarSmallint)));
  XMLToVariant(VariantToXML(FvarSmallint),newVariant);
  Check(VarType(newVariant)=varSmallint, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = SI_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarIntegerWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarInteger)=varInteger, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarInteger)));
  XMLToVariant(VariantToXML(FvarInteger),newVariant);
  Check(VarType(newVariant)=varInteger, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = INT_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarSingleWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarSingle)=varSingle, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarSingle)));
  XMLToVariant(VariantToXML(FvarSingle),newVariant);
  Check(VarType(newVariant)=varSingle, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = FvarSingle, 'Value is lost');
end;

procedure TVariantTestCase.TestVarDoubleWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarDouble)=varDouble, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarDouble)));
  XMLToVariant(VariantToXML(FvarDouble),newVariant);
  Check(VarType(newVariant)=varDouble, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = DOUBLE_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarCurrencyWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarCurrency)=varCurrency, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarCurrency)));
  XMLToVariant(VariantToXML(FvarCurrency),newVariant);
  Check(VarType(newVariant)=varCurrency, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = CURR_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarDateWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarDate)=varDate, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarDate)));
  XMLToVariant(VariantToXML(FvarDate),newVariant);
  Check(VarType(newVariant)=varDate, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = FVarDate, 'Value is lost');
end;

procedure TVariantTestCase.TestVarBooleanWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarBoolean)=varBoolean, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarBoolean)));
  XMLToVariant(VariantToXML(FvarBoolean),newVariant);
  Check(VarType(newVariant)=varBoolean, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = BOOL_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarUnknownWorks;
var
  newVariant : Variant;
begin
  newVariant := FDestIntf;
  Check(VarType(FvarUnknown)=varUnknown, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarUnknown)));
  XMLToVariant(VariantToXML(FvarUnknown),newVariant);
  Check(VarType(newVariant)=varUnknown, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check((FsrcObj.Value = FdestObj.Value), 'Property Value is lost');
end;

procedure TVariantTestCase.TestVarByteWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarByte)=varByte, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarByte)));
  XMLToVariant(VariantToXML(FvarByte),newVariant);
  Check(VarType(newVariant)=varByte, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = BYTE_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarOLEStrWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarOleStr)=varOleStr, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarOleStr)));
  XMLToVariant(VariantToXML(FvarOleStr),newVariant);
  Check(VarType(newVariant)=varOLEStr, 'state is lost current state is:'+VarTypeAsString(VarType(newVariant)));
  Check(newVariant = STR_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestVarStringWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  Check(VarType(FvarString)=varString, 'Source state is incorrect current state is:'+VarTypeAsString(VarType(FvarString)));
  XMLToVariant(VariantToXML(FvarString),newVariant);
  Check(VarType(newVariant)=varString, 'state is lost');
  Check(newVariant = STR_VALUE, 'Value is lost');
end;

procedure TVariantTestCase.TestSimpleVarArrayWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  XMLToVariant(VariantToXML(FSimpleArray),newVariant);
  self.compareSimpleArray(newVariant);
end;

procedure TVariantTestCase.TestSingleDimFloatVarArrayWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  XMLToVariant(VariantToXML(self.createVarArrayOfFloats),newVariant);
  self.compareSingleDimArray(newVariant,self.createVarArrayOfFloats);
end;

procedure TVariantTestCase.TestSingleDimIntegerVarArrayWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  XMLToVariant(VariantToXML(self.createVarArrayOfInteger),newVariant);
  self.compareSingleDimArray(newVariant,self.createVarArrayOfInteger);
end;

procedure TVariantTestCase.TestSingleDimDateVarArrayWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  XMLToVariant(VariantToXML(self.createVarArrayOfDate),newVariant);
  self.compareSingleDimArray(newVariant,self.createVarArrayOfDate);
end;

procedure TVariantTestCase.TestMultidimensionalVarArrayWorks;
var
  newVariant : Variant;
begin
  newVariant := 'Test';
  newVariant := VarArrayCreate([-1,0,0,1,1,4],varVariant);
  newVariant[-1,1,3] := FDestIntf2;
  XMLToVariant(VariantToXML(FMultiDimArray),newVariant);
  self.compareMultiDimArray(newVariant);
end;

initialization
 TestFramework.RegisterTest(TVariantTestCase.Suite);

end.
