unit fHTMLTags;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  xmlworks2, StdCtrls;

type
  {
   copyright 2000 : Thomas Weinert, http://www.subjective.de, info@subjective.de  

   This sample demonstrates the use of a TXMLCollection, which has an
   TXMLCollection as property.  This allows the use of nested collections.
   This is a very powerful feature that allows very complex data sets to be
   stored in XML.


   This sample defines HTMLtags grouped to pages. It can save and read itself
   to/from an xml-file.
  }


  //One HTMLTag
  THTMLTagCollectionItem = class(TXMLCollectionItem)
  private
    FHTMLTagBegin : string;
    FHTMLTagEnd : string;
    FHTMLTagInfo : string;
    FHTMLTagTitle : string;
  published
    property HTMLTagBegin : string read FHTMLTagBegin write FHTMLTagBegin;
    property HTMLTagEnd : string read FHTMLTagEnd write FHTMLTagEnd;
    property HTMLTagInfo : string read FHTMLTagInfo write FHTMLTagInfo;
    property HTMLTagTitle : string read FHTMLTagTitle write FHTMLTagTitle;
  end;

  //Some HTMLTags
  THTMLTagCollection = class(TXMLCollection);

  //One Page with HTMLTags
  THTMLPageCollectionItem = class(TXMLCollectionItem)
  private
    FHTMLPageTitle : string;
    FHTMLTags : THTMLTagCollection;
    procedure SetHTMLTags(Value:THTMLTagCollection);
  public
    constructor Create(Collection: TCollection);override;
    destructor Destroy;override;
  published
    property HTMLPageTitle : string read FHTMLPageTitle write FHTMLPageTitle;
    property HTMLTags : THTMLTagCollection read FHTMLTags write SetHTMLTags;
  end;

  //Some Pages with HTMLTags
  THTMLPageCollection = class(TXMLCollection);

  TfmHTMLTags = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private-Deklarationen }
    iHTMLPages : THTMLPageCollection;
  public
    { Public-Deklarationen }
  end;

var
  fmHTMLTags: TfmHTMLTags;

implementation

{$R *.DFM}

//Automatic create the HTMLTags-Collection, when creating a page
constructor THTMLPageCollectionItem.Create(Collection: TCollection);
begin
  inherited;
  FHTMLTags := THTMLTagCollection.Create(THTMLTagCollectionItem);
end;

//And destroy the subcollection wioth the htmltags
destructor THTMLPageCollectionItem.Destroy;
begin
  FHTMLTags.Free;
  inherited;
end;

//Set the HTMLTags property... maybe duplicate a page :-)
procedure THTMLPageCollectionItem.SetHTMLTags(Value:THTMLTagCollection);
begin
  if FHTMLTags <> Value then
  begin
    FHTMLTags.Assign(Value);
  end;
end;

procedure TfmHTMLTags.Button1Click(Sender: TObject);
var
  MainItem : THTMLPageCollectionItem;
  SubItem : THTMLTagCollectionItem;
begin
  //Pages-Collection exists?
  if Assigned(iHTMLPages) then
  begin
    //Delete all pages with all htmltags
    iHTMLPages.Clear;
    //Create a page
    MainItem := THTMLPageCollectionItem(iHTMLPages.Add);
    //page created?
    if Assigned(MainItem) then
    begin
      //set the properties
      MainItem.HTMLPageTitle := 'Document';
      //Create HTMLtag
      SubItem := THTMLTagCollectionItem(MainItem.HTMLTags.Add);
      //HTMLTag created?
      if Assigned(SubItem) then
      begin
        //Set properties
        SubItem.HTMLTagTitle := '<html>';
        SubItem.HTMLTagBegin := '<html>'+#13;
        SubItem.HTMLTagEnd := #13+'</html>';
        SubItem.HTMLTagInfo := 'Root element of an HTML document';
      end;
      //Create a second HTMLtag
      SubItem := THTMLTagCollectionItem(MainItem.HTMLTags.Add);
      //HTMLTag created?
      if Assigned(SubItem) then
      begin
        //Set properties
        SubItem.HTMLTagTitle := '<body>';
        SubItem.HTMLTagBegin := '<body bgcolor="" text="">'+#13;
        SubItem.HTMLTagEnd := #13+'</body>';
        SubItem.HTMLTagInfo := 'Direct visible part of the'+#13#10+'HTML document';
      end;
    end;
    //Create a second page
    MainItem := THTMLPageCollectionItem(iHTMLPages.Add);
    //page created?
    if Assigned(MainItem) then
    begin
      //set the properties
      MainItem.HTMLPageTitle := 'Font (physical)';
      //Create HTMLtag
      SubItem := THTMLTagCollectionItem(MainItem.HTMLTags.Add);
      //HTMLTag created?
      if Assigned(SubItem) then
      begin
        //Set properties
        SubItem.HTMLTagTitle := '<b> [bold]';
        SubItem.HTMLTagBegin := '<b>';
        SubItem.HTMLTagEnd := '</b>';
        SubItem.HTMLTagInfo := 'Bold text';
      end;
    end;
    //Save and display 
    Memo1.Lines.Text := iHTMLPages.XML;
    iHTMLPages.SaveToFile(ChangeFileExt(Application.ExeName,'.xml'));
  end;
end;

procedure TfmHTMLTags.FormCreate(Sender: TObject);
begin
  iHTMLPages := THTMLPageCollection.Create(THTMLPageCollectionItem);
end;

procedure TfmHTMLTags.FormDestroy(Sender: TObject);
begin
  iHTMLPages.Free;
end;

procedure TfmHTMLTags.Button2Click(Sender: TObject);
var
  Subs, i : integer;
begin
  //Load xml data from a file.
  iHTMLPages.LoadFromFile(ChangeFileExt(Application.ExeName,'.xml'));
  //count pages
  label1.Caption := 'Parsed Pages: '+IntToStr(iHTMLPages.Count);
  //count all htmltags from all pages
  subs := 0;
  for i := 0 to iHTMLPages.Count-1 do
  begin
    subs := subs + (THTMLPageCollectionItem(iHTMLPages.Items[i])).HTMLTags.Count;
  end;
  label2.Caption := 'Parsed HTMLtags: '+IntToStr(subs);

  memo1.Lines.Text := iHTMLPages.XML;
end;

end.
