unit Tester;

(*

 This is a real sample from a server used to feed real time weather information
 to a client written in FlashMX.  This server receives several thousand updates
 per hour from a satellite feed then sends them out to clients "real time".

 Obviously all the logic for connecting to the satellite and clients has been
 removed for simplicity.

 This example will show the speed that XMLWorks can generate and read XML.

*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, xmlworks2, contnrs, typinfo, Db, DBTables,
  ADODB, ADOInt, umdlist;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

type
  TStationItem = class(TXMLCollectionItem)
  protected
    f_StationName: string;
    f_temp: string;
    f_city: string;
    f_state: string;
    f_country: string;
    f_dttmobs: string;
    f_latitd: string;
    f_longtd: string;
    f_visib_sm: string;
    f_precip: string;
    f_clouds: string;
    f_windDirMph: string;
    f_dateMsg: string;

    f_elev_ft: string;
    f_dewpt_f: string;
    f_press_inhg: string;
    f_relHumPct: string;
    f_heatndx_f: string;
    f_windchl_f: string;
    f_obsType: string;

    f_MinTemp: integer;
    f_MaxTemp: integer;
    f_MinTime: string;
    f_MaxTime: string;
    f_hdd: integer;
    f_cdd: integer;

    f_LastMessageDTM: TDateTime;
  public
    class function getTagName : String; override;
    procedure Randomize;
  published
    property name: string read f_StationName write f_StationName;
    property temp: string read f_temp write f_temp;
    property city: string read f_city write f_city;
    property state: string read f_state write f_state;
    property country: string read f_country write f_country;
    property dttmobs: string read f_dttmobs write f_dttmobs;
    property latitd: string read f_latitd write f_latitd;
    property longtd: string read f_longtd write f_longtd;
    property elev_ft: string read f_elev_ft write f_elev_ft;
    property visib_sm: string read f_visib_sm write f_visib_sm;
    property dewpt_f: string read f_dewpt_f write f_dewpt_f;
    property press_inhg: string read f_press_inhg write f_press_inhg;
    property precip: string read f_precip write f_precip;
    property clouds: string read f_clouds write f_clouds;
    property windDirMph: string read f_windDirMph write f_windDirMph;
    property relHumPct: string read f_relHumPct write f_relHumPct;
    property heatndx_f: string read f_heatndx_f write f_heatndx_f;
    property windchl_f: string read f_windchl_f write f_windchl_f;
    property obsType: string read f_obsType write f_obsType;
    property dateMsg: string read f_dateMsg write f_dateMsg;
    property mintemp: integer read f_MinTemp write f_MinTemp;
    property maxtemp: integer read f_MaxTemp write f_MaxTemp;
    property mintime: string read f_MinTime write f_MinTime;
    property maxtime: string read f_MaxTime write f_MaxTime;

    property hdd: integer read f_hdd write f_hdd;
    property cdd: integer read f_cdd write f_cdd;
  end;

  TStationCollection = class(TXMLCollection)
  end;


procedure TForm1.Button1Click(Sender: TObject);
var
  I: Integer;
  TempTkCt: DWord;
  Stations: TStationCollection;
  TempXML: String;
begin

  Stations := TStationCollection.Create(TStationItem);


  // Creating 1000 records
  Memo1.Lines.Add('Adding 10000 records');
  Application.ProcessMessages();
  TempTkCt := GetTickCount();
  for I := 1 to 1000 do    // Iterate
    TStationItem(Stations.Add).Randomize();

  Memo1.Lines.Add('Took ' + IntToStr(GetTickCount() - TempTkCt) + ' ticks');
  Application.ProcessMessages;

  // Saving to a file
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Saving to "test.xml"');
  Application.ProcessMessages();
  TempTkCt := GetTickCount();

  Stations.SaveToFile('test.xml');

  Memo1.Lines.Add('Took ' + IntToStr(GetTickCount() - TempTkCt) + ' ticks');
  Application.ProcessMessages;


  // Read XML
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Reading Entire Collection');
  Application.ProcessMessages();
  TempTkCt := GetTickCount();

  TempXML := Stations.XML;

  Memo1.Lines.Add(IntToStr(Length(TempXML) div 1024) + ' kbytes');
  Memo1.Lines.Add('Took ' + IntToStr(GetTickCount() - TempTkCt) + ' ticks');
  Application.ProcessMessages;

  // Read Single XML
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Reading Single Item');
  Application.ProcessMessages();
  TempTkCt := GetTickCount();

  TempXML := Stations[0].XML;

  Memo1.Lines.Add(IntToStr(Length(TempXML)) + ' bytes');
  Memo1.Lines.Add('Took ' + IntToStr(GetTickCount() - TempTkCt) + ' ticks');
  Application.ProcessMessages;

  ShowMessage(TempXML);

  // Clearing Data
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Clearing Collection');
  Application.ProcessMessages();
  TempTkCt := GetTickCount();

  Stations.Clear;

  Memo1.Lines.Add('Took ' + IntToStr(GetTickCount() - TempTkCt) + ' ticks');
  Application.ProcessMessages;

  // Loading from a file
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Loading from file');
  Application.ProcessMessages();
  TempTkCt := GetTickCount();

  Stations.LoadFromFile('test.xml');

  Memo1.Lines.Add('Loaded ' + IntToStr(Stations.Count) + ' records');
  Memo1.Lines.Add('Took ' + IntToStr(GetTickCount() - TempTkCt) + ' ticks');
  Application.ProcessMessages;

end;

{ TStationItem }

class function TStationItem.getTagName: String;
begin
  // This defines the "tagname" for each Item.  In other words the XML will look like:
  //  <Station></Station>

  result := 'Station';
end;


function RandomString(const CharCt: integer): string;
var
  I: Integer;
begin
  result := '';
  for I := 1 to CharCt do    // Iterate
    result := result + Char(Random(26) + Ord('A'));
end;

function RandomIntString(const DigitCt: integer): string;
var
  I: Integer;
begin
  result := '';
  for I := 1 to DigitCt do    // Iterate
    result := result + Char(Random(10) + Ord('0'));
end;

procedure TStationItem.Randomize;
begin
  name := RandomString(3);
  temp := RandomIntString(2);
  city := RandomString(4);
  state := RandomString(2);
  country := RandomString(3);
  dttmobs := RandomString(10);
  latitd := RandomIntString(3);
  longtd := RandomIntString(3);
  elev_ft := RandomIntString(3);
  visib_sm := RandomIntString(3);
  dewpt_f := RandomIntString(3);
  press_inhg := RandomIntString(3);
  precip := RandomIntString(3);
  clouds := RandomIntString(3);
  windDirMph := RandomIntString(3);
  relHumPct := RandomIntString(3);
  heatndx_f := RandomIntString(3);
  windchl_f := RandomIntString(3);
  obsType := RandomIntString(3);
  dateMsg  := RandomIntString(3);

  mintemp := Random(100);
  maxtemp := Random(100);
  mintime := DateTimeToStr(Now);
  maxtime := DateTimeToStr(Now);

  hdd := Random(100);
  cdd := Random(100);
end;

end.


