unit XMLWorks2;

{
  XMLWorks2

  Version   : 2.5 devel
  Date      : 8/05/2002
    
  Descripton:
  For Latest Versions:
          http://www.DelphiHome.com/xml (North America)
          http://www.XMLWorks.de (German Mirror)


  Author: Marc Bir
          http://www.DelphiHome.com
          xmlworks@delphihome.com


  Author: Thomas Weinert (subjective)
          http://www.subjective.de
          info@subjective.de

  Author: Sancho Fock (thsfock)
          www.thsfock.de
          sancho@thsfock.de

  Thanks to:
    Chad Hower
      www.AtoZedSoftware.com
     
      
          
  Major Additions to Version 2

  - Correctly supports nested elements of the same name
  - Added Several new objects, support for IXMLWorksObject interface
  - Reorganized code to allow quick creation of new custom objects for streaming
  - Added support for Interfaced Properties
  - Fixed Variant Support
  - Changed ancestor of XMLObject to TObject
  - Supports Delphi through Version 6



  Description

    XMLWorks is a collection of OpenSource utilities designed to help Delphi developers create XML
    documents.

    XMLWorks is in no way designed to be an all encompassing package for XML, but it does offer
    many features that should help Delphi developers working with XML.

    The basic idea behind XMLWorks is to use Delphi Objects to generate DTD and entity
    information.  All you have to do is create a descendant class of one of the XML classes creating
    published properties for the data you want converted to XML.  To convert back from XML the 
    property
    must not be read only.

    In general you don't need to do more than declare a descendant class of TXMLCollection named
    something like TAuthorCollection.  When generating the DTD it will automatically use the name
    'Author' (whatever text is between the 'T' and 'Collection') + '-list'.

    Most of the classes have basic rules for how the class name translates to a tag name, however 
    this default behaviour can be changed by overriding the class function getTagName.
    
}




interface
uses
    Classes     // TPersistent
  , SysUtils    // Exceptions
  , Graphics    // TColor utilities
  , TypInfo;    // RTTI Stuff


type

{:
 We must turn TypeInfo on here to ensure all classes or created with RTTI (otherwise TInterfacedXMLObject would not)
 RTTI Data is the foundation of XMLWorks
}

{$TYPEINFO ON}


{:  
  XMLString is a special type that tells the streaming mechanism not to encode the contents.
} 
  XMLString = type ansistring;

{:  
  XMLString is a special type that tells the streaming mechanism to use MIME encoding.
} 
  XMLMIMEString = type ansistring;

{:  
  XMLRGBTColor is a special type that tells the streaming mechanism to use HEX encoding and to swap 
  the R and B values.  This allows users to edit the XML using standard RGB values, and programmers
  to directly use the value as a TColor.
} 
  XMLRGBTColor = type integer;

  IXMLWorksObject = interface (IUnknown)
    ['{FF809F5B-0856-4BFB-A1D0-7271DFD306A5}']
    function getDTDElements: string;
    function getDTDSequence: string;
    function GetElementText: string;
    function GetXML: string;
    procedure LoadFromFile(FileName:string);
    procedure SaveToFile(FileName:string);
    procedure SetElementText(const p_sXML: string);
    procedure SetXML(const p_sXML: string);
    property ElementText: string read GetElementText write SetElementText;
    property XML: string read GetXML write SetXML;
  end;
  
  TXMLInterfacedObject = class (TInterfacedObject, IXMLWorksObject)
  protected
    function GetElementText: string; virtual;
    function GetXML: string;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetXML(const p_sXML: string);
  public
    constructor Create(p_sXML: String = ''); overload; virtual;
    function getDTDElements: string;
    function getDTDSequence: string;
    class function getTagName: string; virtual;
    procedure LoadFromFile(FileName:string);
    procedure SaveToFile(FileName:string);
    property ElementText: string read GetElementText write SetElementText;
    property XML: string read GetXML write SetXML;
  end;
  
  TXMLObject = class (TObject)
  protected
    function GetElementText: string; virtual;
    function GetXML: string;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetXML(const p_sXML: string);
  public
    constructor Create(p_sXML: String = ''); overload; virtual;
    function GetDTD: string;
    function getDTDElements: string;
    function getDTDSequence: string;
    class function getTagName: string; virtual;
    procedure LoadFromFile(FileName:string);
    procedure SaveToFile(FileName:string);
    property ElementText: string read GetElementText write SetElementText;
    property XML: string read GetXML write SetXML;
  end;
  
  TXMLList = class (TStringList)
  protected
    function GetElementText: string; virtual;
    function GetXML: string;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetXML(const p_sXML: string);
  public
    function getDTDElements: string;
    function getDTDSequence: string;
    class function getItemTagName: string; virtual;
    class function getTagName: string; virtual;
    property ElementText: string read GetElementText write SetElementText;
    property XML: string read GetXML write SetXML;
  end;
  
  TXMLCollectionItem = class (TCollectionItem)
  protected
    function GetElementText: string; virtual;
    function GetXML: string;
    function getXMLAsProperties: string;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetXML(const p_sXML: string);
  public
    function getDTDElements: string;
    function getDTDSequence: string;
    class function getTagName: string; virtual;
    property ElementText: string read GetElementText write SetElementText;
    property XML: string read GetXML write SetXML;
  end;
  

  TXMLCollectionItemClass = class of TXMLCollectionItem;
  TCollectionStringCompare = function (const Value: String; const Item2: TXMLCollectionItem): Integer;
  TCollectionIntegerCompare = function (const Value: Integer; const Item2: TXMLCollectionItem): Integer;
  TXMLCollection = class (TCollection)
  protected
    function GetElementText: string; virtual;
    function GetXML: string; 
    function GetXMLCollectionItem(Index:Integer): TXMLCollectionItem;
    procedure SetElementText(const p_sXML: string); virtual;
    procedure SetXML(const p_sXML: string);
    procedure SetXMLCollectionItem(Index:Integer; Value: TXMLCollectionItem);
  public
    constructor Create(ItemClass: TXMLCollectionItemClass); virtual;
    function Add: TXMLCollectionItem;
    function FindInt(Value: Integer; Compare: TCollectionIntegerCompare): 
            TXMLCollectionItem;
    function FindStr(Value: String; Compare: TCollectionStringCompare): 
            TXMLCollectionItem;
    function getDTDElements: string;
    function getDTDSequence: string;
    function getItemTagName: string; virtual;
    function getPropertiesXML: string;
    function getSubsetElementText(Start, Ct: Integer): string;
    class function getTagName: string; virtual;
    procedure LoadFromFile(FileName:string);
    procedure SaveToFile(FileName:string);
    procedure Sort(Compare: TListSortCompare);
    property ElementText: string read GetElementText write SetElementText;
    property Items[Index:Integer]: TXMLCollectionItem read GetXMLCollectionItem 
            write SetXMLCollectionItem; default;
    property XML: string read GetXML write SetXML;
  end;
  

  EXMLException = class (Exception)
  end;
  
  EXMLInterfaceException = class (EXMLException)
  end;
  
  EXMLIOException = class (EXMLException)
  end;
  


function ObjectToXMLElements(const aObject:TObject): String;
function ObjectToXMLProperties(const aObject:TObject): String;

function StrToXML(const p_sXML:string):string;
function XMLtoStr(const p_sXML:string):string;

function getTagContent(const p_sXML, TargetTagName: String): string;

procedure setPropAsString(Instance: TObject; PropInfo: PPropInfo; const value : 
        string);

procedure SaveStringToFile(const Str, FileName:string);

const
  kernel = 'kernel32.dll';

function InterlockedIncrement(var Addend: Integer): Integer; stdcall;
  external kernel name 'InterlockedIncrement';

function InterlockedDecrement(var Addend: Integer): Integer; stdcall;
  external kernel name 'InterlockedDecrement';

function getPropertyList(const ClassInfo: Pointer): TList;

function getObjectDTDElements(const aObject:TObject): string;

function getObjectDTDSequence(const aObject:TObject): string;

function LoadStringFromFile(const FileName:string): string;

procedure setXMLObject(Instance: TObject; p_sXML: AnsiString);

function InterfaceToXML(Unk : IUnknown): string;

procedure XMLtoInterface(pXML : string; unk : IUnknown);

function FastParseTag(const Source,Start,Stop:AnsiString;var Index:Integer): 
        AnsiString;

function FastParseTagXML(const Source, Tag : AnsiString;var Index:Integer):
        AnsiString;

function FastParse(const Source: AnsiString; const Delim: Char; var Index:
        Integer): AnsiString;

function FastToken(const Source: AnsiString; const Delim: Char; Index:
        Integer): AnsiString;

function SwapRandB(inRGB: XMLRGBTColor): TColor;

function LoadStringFromStream(Stream:TStream): string;

procedure SaveStringToStream(const Str:string; Stream:TStream);

var
  // Global Boolean specifies the sort order of generated XML strings, False uses the declared order
  gb_XMLAlphaSortProperties : Boolean = true;
  // Global Boolean causes an exception to be raised when generating XML for a class property that is not supported
  gb_XMLRaiseInvalidPropertyClass : Boolean = false;


implementation

uses
   Windows     // RGB
  , FastStrings
  , FastStringFuncs
  , XMLVariants;


const
{MMWIN:ENDEXPAND}
  cs_XMLPrologTag = '?xml version = "1.0"?';
  cs_XMLProlog = '<' + cs_XMLPrologTag + '>';
  cs_XMLLIST_itemTagName = 'value';
  cs_CDATA_Descriptor = '<![CDATA[';
  cs_CDATA_Descriptor_Close = ']]>';
  cs_DocTypeHeader = '<!DOCTYPE ';
  cs_DocTypeHeader_Close = ']>';
  cs_ElementHeader = '<!ELEMENT ';

{MMWIN:STARTEXPAND}

function SwapRandB(inRGB: XMLRGBTColor): TColor;
asm
  BSwap EAX
  ShR   EAX, 8
end;

{:

}
function getPropertyList(const ClassInfo: Pointer): TList;
var
  i, iPropCount: integer;
  PropList: PPropList;
begin
  result := TList.Create;
  try
    iPropCount := GetTypeData(ClassInfo)^.PropCount;
    if iPropCount > 0 then
    begin
      GetMem(PropList, iPropCount * SizeOf(Pointer));
      try
        iPropCount := GetPropList(ClassInfo, tkProperties, PropList);

        if gb_XMLAlphaSortProperties then
        begin // Alpha Order
          for i := 0 to (iPropCount-1) do
            result.Add(PropList^[i]);
        end
        else // Declared Order
        begin
          result.Count := iPropCount;
          { TODO : Weird issue to work around... needs more research }
          for i := 0 to (iPropCount-1) do
            if (PropList^[i].NameIndex > result.Count-1) then
              result.Count := PropList^[i].NameIndex+1;

          for i := 0 to (iPropCount-1) do
            Result[PropList^[i].NameIndex] := (PropList^[i]);

          for i := result.Count-1 downto 0 do
            if not assigned(result[i]) then
              result.Delete(i);


        end;

      finally
        FreeMem(PropList);
      end;
    end;
  except
    Result.Free;
    Raise
  end;
end;

function StrToXML(const p_sXML:string):string;
const
  ValidCharTable = (['a'..'z','A'..'Z','0'..'9','-',' ',',','.','\','/','_','[',']','!',';',':','=']);
var
  c : char;
  i : integer;
begin
  result := p_sXML;
  for i := Length(result) downto 1 do
  begin
    c := result[i];
    if not (c in ValidCharTable)  then
    begin
      result[i] := '&';
      Insert('#x' + IntToHex(Ord(c),2) + ';', result,i+1);
    end;
  end;
end;

function XMLToStr(const p_sXML:string): string;
var
  hexno : string;
  i, h : integer;
begin
  Result := p_sXML;
  if AnsiSameText(Copy(Result, 0, 9), cs_CDATA_Descriptor) then
  begin
    // Trims the Open and close tags off the string and returns it. 
    Delete(Result, 1, 9);
    Delete(Result, Length(Result)-2, 3);
    exit;
  end;

  for i := Length(Result)-5 downto 1 do begin
    if (Result[i] = '&') and (Length(Result) >= i+5) then
    begin
      if (Result[i+1] = '#') and (Result[i+2] = 'x') then
      begin
        hexno := Result[i+3]+Result[i+4];
        // Must add '0x' to force Hex type, otherwise StrToInt assumes base 10
        h := StrToInt('0x' + hexno);
        Delete(Result, i, 5);
        Result[i] := chr(h);
      end;
    end;
  end;
  Result := Trim(Result);
end;

{: 
  AssignIntf/GetIntfProp are here to provide the most basic access to
  published properties and should be immediately suspect if you are having any
  problems with XML classes that have published interfaces.  Although no known 
  issues have surfaced as of yet.  NOTE: This has only been tested with Delphi
  5 and 6
}

procedure AssignIntf(var Dest: IUnknown; const Source: IUnknown);
begin
  Dest := Source;
end;

function GetIntfProp(Instance: TObject; PropInfo: PPropInfo): IUnknown;
asm
        { ->    EAX Pointer to instance         }
        {       EDX Pointer to property info    }
        {       ECX Pointer to result interface }

        PUSH    ESI
        PUSH    EDI
        MOV     EDI,EDX

        MOV     EDX,[EDI].TPropInfo.Index       { pass index in EDX }
        CMP     EDX,$80000000
        JNE     @@hasIndex
        MOV     EDX,ECX                         { pass value in EDX }
@@hasIndex:
        MOV     ESI,[EDI].TPropInfo.GetProc
        CMP     [EDI].TPropInfo.GetProc.Byte[3],$FE
        JA      @@isField
        JB      @@isStaticMethod

@@isVirtualMethod:
        MOVSX   ESI,SI                          { sign extend slot offset }
        ADD     ESI,[EAX]                       { vmt + slot offset }
        CALL    DWORD PTR [ESI]
        JMP     @@exit

@@isStaticMethod:
        CALL    ESI
        JMP     @@exit

@@isField:
  AND  ESI,$00FFFFFF
  ADD  EAX, ESI
  MOV  EDX,[EAX]
  MOV  EAX, ECX

  CALL  AssignIntf

@@exit:
        POP     EDI
        POP     ESI
end;

function InterfaceToXML(Unk : IUnknown): string;
var
  Intf : IXMLWorksObject;
begin
  if Assigned(Unk) then
  begin
    Unk.QueryInterface(IXMLWorksObject, Intf);
    if assigned(Intf) then
      result := Intf.ElementText
//    else if gb_XMLRaiseInvalidPropertyClass then 
//      raise EXMLInterfaceException.Create('Invalid Interface Property (' + PropInfo.Name + ')');
  end
end;

procedure XMLtoInterface(pXML : string; unk : IUnknown);
var
  Intf : IXMLWorksObject;
begin
  if Assigned(Unk) then
  begin
    Unk.QueryInterface(IXMLWorksObject, Intf);
    if assigned(Intf) then
      Intf.ElementText := pXML;
  end
end;


function GetPropAsString(const Instance: TObject; const PropInfo: PPropInfo): string;
var
  ObjectProp : TObject;
  Intf: IXMLWorksObject;

begin
  result := '';
  case PropInfo^.PropType^.Kind of
    tkString,
    tkLString,
    tkWString:
      if AnsiSameText(PropInfo^.PropType^.Name, 'XMLString') then
        result := GetStrProp(Instance, PropInfo)
      else if AnsiSameText(PropInfo^.PropType^.Name, 'XMLMIMEString') then
        result := Base64Encode(GetStrProp(Instance, PropInfo))
      else
        result := StrToXML(GetStrProp(Instance, PropInfo));

    tkInt64: result := IntToStr(GetInt64Prop(Instance, PropInfo));
    tkSet,
    tkInteger: 
      if AnsiSameText(PropInfo^.PropType^.Name, 'XMLRGBTColor') then
        result := '$' + IntToHex(SwapRandB(GetOrdProp(Instance, PropInfo)), 6)
      else
        result := IntToStr(GetOrdProp(Instance, PropInfo));
    tkFloat:
      if AnsiSameText(PropInfo^.PropType^.Name, 'TDateTime') then
        result := DateTimeToStr(GetFloatProp(Instance, PropInfo))
      else if AnsiSameText(PropInfo^.PropType^.Name, 'TTime') then
        result := TimeToStr(GetFloatProp(Instance, PropInfo))
      else if AnsiSameText(PropInfo^.PropType^.Name, 'TDate') then
        result := DateToStr(GetFloatProp(Instance, PropInfo))
      else
        result := FloatToStr(GetFloatProp(Instance, PropInfo));
    tkVariant: result := VariantToXML(GetVariantProp(Instance, PropInfo));
    tkChar,
    tkWChar: result := StrToXML(Chr(GetOrdProp(Instance, PropInfo)));
    tkEnumeration: result := StrToXML(GetEnumName( PropInfo^.PropType^, GetOrdProp(Instance, PropInfo)));
    tkClass:
      begin
        ObjectProp := TObject(GetOrdProp(Instance, PropInfo));
        if Assigned(ObjectProp) then
        begin
          if ObjectProp.GetInterface(IXMLWorksObject, Intf) then
            result := Intf.ElementText
          else if (ObjectProp is TXMLCollection) then
            result := TXMLCollection(ObjectProp).ElementText
          else if (ObjectProp is TXMLCollectionItem) then
            result := TXMLCollectionItem(ObjectProp).ElementText
          else if (ObjectProp is TXMLObject) then
            result := TXMLObject(ObjectProp).ElementText
          else if (ObjectProp is TXMLList) then
            result := TXMLList(ObjectProp).ElementText   
          else if (ObjectProp is TStrings) then
            result := StrToXML((ObjectProp as TStrings).CommaText)
          else
            if gb_XMLRaiseInvalidPropertyClass then
              raise EXMLException.Create('Invalid Class Property (' + PropInfo.Name + ')');
        end;
      end;
    tkInterface: 
      result := InterfaceToXML(GetIntfProp(Instance, PropInfo));
  end;
end;


procedure setPropAsString(Instance: TObject; PropInfo: PPropInfo; const value : 
        string);
var
  ObjectProp : TObject;
  Intf: IXMLWorksObject;
  vTemp : variant;  
begin
  //  No property          
  if (PropInfo = Nil) OR (value = '') or
    // a read only simple type
    ((PropInfo^.SetProc = NIL) and not (PropInfo^.PropType^.Kind in [tkClass, tkInterface])) then
    exit;
    

  case PropInfo^.PropType^.Kind of
    tkString,
    tkLString,
    tkWString: 
      if AnsiSameText(PropInfo^.PropType^.Name, 'XMLString') then
        SetStrProp(Instance, PropInfo, Value)
      else if AnsiSameText(PropInfo^.PropType^.Name, 'XMLMIMEString') then
        SetStrProp(Instance, PropInfo, Base64Decode(Value))
      else
        SetStrProp(Instance, PropInfo, XMLToStr(Value));
    tkSet,
    tkInteger: 
      if AnsiSameText(PropInfo^.PropType^.Name, 'XMLRGBTColor') then
        SetOrdProp(Instance, PropInfo, SwapRandB(StrToInt(XMLToStr(Value))))
      else
        SetOrdProp(Instance, PropInfo, StrToInt(XMLToStr(Value)));
    tkFloat:
      if AnsiSameText(PropInfo^.PropType^.Name, 'TDateTime')  then
        SetFloatProp(Instance, PropInfo, StrToDateTime(XMLToStr(Value)))
      else if AnsiSameText(PropInfo^.PropType^.Name, 'TTime')  then
        SetFloatProp(Instance, PropInfo, StrToTime(XMLToStr(Value)))
      else if AnsiSameText(PropInfo^.PropType^.Name, 'TDate')  then
        SetFloatProp(Instance, PropInfo, StrToDate(XMLToStr(Value)))
      else
        SetFloatProp(Instance, PropInfo, StrToFloat(XMLToStr(Value)));

    tkVariant: 
      begin
        vTemp := GetVariantProp(Instance,PropInfo);
        XMLToVariant(value,vTemp);
        SetVariantProp(Instance, PropInfo, vTemp);
      end;
                   
    tkInt64: SetInt64Prop(Instance, PropInfo, StrToInt64(XMLToStr(Value)));

    tkChar,
    tkWChar: SetOrdProp(Instance, PropInfo, Ord(XMLToStr(Value)[1]));

    tkEnumeration: SetOrdProp(Instance, PropInfo, GetEnumValue( PropInfo^.PropType^, XMLToStr(Value)));
    tkClass :
      begin
        try
          ObjectProp := TObject(GetOrdProp(Instance, PropInfo));
          if Assigned(ObjectProp) then
          begin
            if ObjectProp.GetInterface(IXMLWorksObject, Intf) then
              Intf.ElementText := Value
            else if (ObjectProp is TXMLCollection) then
              TXMLCollection(ObjectProp).ElementText := Value
            else if (ObjectProp is TXMLCollectionItem) then
              TXMLCollectionItem(ObjectProp).ElementText := Value
            else if (ObjectProp is TXMLObject) then
              TXMLObject(ObjectProp).ElementText := Value
            else if (ObjectProp is TXMLList) then
              TXMLList(ObjectProp).ElementText := Value   
            else if (ObjectProp is TStrings) then
              TStrings(ObjectProp).CommaText := XMLToStr(Value)
          end;
        except
          on e: Exception do
            raise EXMLException.Create('(' + e.Message + ')Error with property - ' + PropInfo^.Name);
        end;
      end;
    tkInterface: 
      XMLtoInterface(Value,GetIntfProp(Instance, PropInfo));
{
  Types not supported :
    tkRecord
    tkArray
    tkDynArray
    tkMethod
    tkUnknown
}
  end;

end;


function ObjectToXMLElements(const aObject:TObject): String;
var
  i : Integer;
  s : string;
  StringList : TStringList;
  Props: TList;
begin
  result := '';
  StringList := TStringList.Create;
  try
    Props := GetPropertyList(aObject.ClassInfo);
    try
      for i := 0 to Props.Count-1 do
      begin
        s := GetPropAsString(AObject, PPropInfo(Props.Items[i]));
          
        StringList.Add('<' + PPropInfo(Props.Items[i]).Name + '>' + s + '</' + PPropInfo(Props.Items[i]).Name + '>');
      end;
      result := StringList.Text;
    finally
      Props.Free;
    end;
  finally
    StringList.Free;
  end;
end;


function ObjectToXMLProperties(const aObject:TObject): String;
var
  i : Integer;
begin
  result := ' ';
  with GetPropertyList(aObject.ClassInfo) do
  try
    for i := 0 to Count-1 do
      result := result + PPropInfo(Items[i]).Name + '="' + GetPropAsString(AObject, PPropInfo(Items[i])) + '" ';
  finally
    Free;
  end;
end;

function getObjectDTDSequence(const aObject:TObject): string;
var
  i : Integer;
begin
  result := '';
  with GetPropertyList(aObject.ClassInfo) do
  try
    if Count < 1 then
      exit;
      
    for i := 0 to Count-2 do
      result := result + PPropInfo(Items[i]).Name + ', ';
    result := result + PPropInfo(Items[Count-1]).Name;
  finally
    Free;
  end;
end;

function getPropertyDTD(Instance: TObject; PropInfo: PPropInfo) : string;
var
  ObjectProp : TObject;
  Intf: IXMLWorksObject;
begin
  case PropInfo^.PropType^.Kind of
    tkString,
    tkLString,
    tkWString,
    tkInt64,
    tkInteger,
    tkFloat,
    tkVariant,
    tkChar,
    tkWChar,
    tkEnumeration : result := '#PCDATA';
    tkClass:
      begin
        ObjectProp := TObject(GetOrdProp(Instance, PropInfo));
        if Assigned(ObjectProp) then
        begin
          if ObjectProp.GetInterface(IXMLWorksObject, Intf) then
            result := Intf.getDTDSequence
          else if (ObjectProp is TXMLCollection) then
            result := TXMLCollection(ObjectProp).getDTDSequence
          else if (ObjectProp is TXMLCollectionItem) then
            result := TXMLCollectionItem(ObjectProp).getDTDSequence
          else if (ObjectProp is TXMLObject) then
            result := TXMLObject(ObjectProp).getDTDSequence
          else if (ObjectProp is TXMLList) then
            result := TXMLList(ObjectProp).getDTDSequence
          else
            if gb_XMLRaiseInvalidPropertyClass then
              raise EXMLException.Create('Invalid Class Property (' + PropInfo.Name + ')');
        end;
      end;
    else
      Result := '';
  end;
end;

function getPropertyDTDElements(Instance: TObject; PropInfo: PPropInfo) : string;
var
  ObjectProp : TObject;
  Intf: IXMLWorksObject;
begin
  case PropInfo^.PropType^.Kind of
    tkClass:
      begin
        ObjectProp := TObject(GetOrdProp(Instance, PropInfo));
        if Assigned(ObjectProp) then
        begin
          if ObjectProp.GetInterface(IXMLWorksObject, Intf) then
            result := Intf.getDTDElements
          else if (ObjectProp is TXMLCollection) then
            result := TXMLCollection(ObjectProp).getDTDElements
          else if (ObjectProp is TXMLCollectionItem) then
            result := TXMLCollectionItem(ObjectProp).getDTDElements
          else if (ObjectProp is TXMLObject) then
            result := TXMLObject(ObjectProp).getDTDElements
          else if (ObjectProp is TXMLList) then
            result := TXMLList(ObjectProp).getDTDElements
          else
            if gb_XMLRaiseInvalidPropertyClass then
              raise EXMLException.Create('Invalid Class Property (' + PropInfo.Name + ')');
        end;
      end;
    else
      Result := '';
  end;
end;

function getObjectDTDElements(const aObject:TObject): String;
var
  i : Integer;
  s : string;
  StringList : TStringList;
  Props: TList;
begin
  result := '';
  StringList := TStringList.Create;
  try
    Props := GetPropertyList(aObject.ClassInfo);
    try
      for i := 0 to Props.Count-1 do
      begin
        StringList.Add(cs_ElementHeader + PPropInfo(Props.Items[i]).Name + '(' + getPropertyDTD(AObject, PPropInfo(Props.Items[i])) + ')>');

        s := getPropertyDTDElements(AObject, PPropInfo(Props.Items[i]));
        if s <> '' then
          StringList.Add(s);
      end;
      result := StringList.Text;
    finally
      Props.Free;
    end;
  finally
    StringList.Free;
  end;
end;

function FastPosChecked(const aSourceString, aFindString : String;
  const aSourceLen, aFindLen, StartPos : Integer) : Integer;
var
  Offset, MaxPos : integer;
  c : Char;
begin
  Offset := StartPos;
  MaxPos := aSourceLen - aFindLen -1;
  c := aFindString[aFindLen];
  repeat
    Result := FastStrings.FastPos(aSourceString, aFindString, aSourceLen, aFindLen, Offset);
    if (Result > 0) and (Result <= MaxPos) then
    begin
      if aSourceString[Result+aFindLen-1] = c then
      begin
        Offset := -1;
      end else begin
        Offset := Result+aFindLen;
      end;
    end;
  until (Result = 0) or (Offset = -1) or (Result > MaxPos);
end;

function FastParseTag(const Source, Start, Stop: AnsiString; var Index: Integer): AnsiString;
var
  StartLen, StopLen, SourceLen: Integer;
  StartIndex, StopIndex: Integer;
begin
  StartIndex := 0;
  Result := '';
  SourceLen := Length(Source);
  StartLen := Length(Start);
  StopLen := Length(Stop);

  if (Index > 0) and (Index < SourceLen) and (StartLen > 0) and (StopLen > 0) then
  begin
    StartIndex := FastPosChecked(Source, Start, SourceLen, StartLen, Index);
    if StartIndex > 0 then
    begin
      Inc(StartIndex, StartLen);
      StopIndex := FastPosChecked(Source, Stop, SourceLen, StopLen, StartIndex);
      if StopIndex > 0 then
      begin
        Result := Copy(Source, StartIndex, StopIndex - StartIndex);
        StartIndex := StopIndex + StopLen;
      end
      else
        StartIndex := 0;
    end;
  end;
  Index := StartIndex;
end;


function FastParse(const Source: AnsiString; const Delim: Char; var Index: 
        Integer): AnsiString;
var
  SourceLen: Integer;
  NewIndex: Integer;
  StopIndex: Integer;
begin
  Result := '';
  SourceLen := Length(Source);

  if (Index > 0) and (Index <= SourceLen) then
  begin
    StopIndex := FastPos(Source, Delim, SourceLen, 1, Index);
    if StopIndex = 0 then
    begin
      StopIndex := SourceLen + 1;
      NewIndex := 0; 
    end
    else
      NewIndex := StopIndex + 1;
    
    result := Copy(Source, Index, StopIndex-Index); 
  end
  else
    NewIndex := 0;

  Index := NewIndex;
end;

function FastToken(const Source: AnsiString; const Delim: Char; Index: 
        Integer): AnsiString;
var
  SourceLen: Integer;
  StartIndex, StopIndex: Integer;
begin
  Result := '';
  SourceLen := Length(Source);

  if (Index > 0) and (Index <= SourceLen) and (Source[Index] <> Delim) then
  begin
    if (Index = SourceLen) then
    begin
      StartIndex := FastPosBack(Source, Delim, SourceLen, 1, Index) + 1;
      
      StopIndex := SourceLen;
    end
    else if (Index = 1) then
    begin
      StartIndex := 1; 
      StopIndex := FastPos(Source, Delim, SourceLen, 1, Index) - 1;
    end
    else
    begin
      StartIndex := FastPosBack(Source, Delim, SourceLen, 1, Index) + 1; 
      StopIndex := FastPos(Source, Delim, SourceLen, 1, Index) - 1;  
    end;

    if StartIndex = 0 then
      StartIndex := 1;
    if StopIndex = 0 then
      StopIndex := SourceLen;

    Result := Copy(Source, StartIndex, StopIndex - StartIndex + 1);
  end;
end;

function FastParseTagXML(const Source, Tag: AnsiString; var Index: Integer): 
        AnsiString;
var
  NestLevel: Integer;
  StartTag, StopTag: AnsiString;
  StartLen, StopLen, SourceLen: Integer;
  StartIndex, StopIndex: Integer;
begin
  SourceLen := Length(Source);
  StartIndex := Index;
  Index := 0;
  result := '';
  if (StartIndex > 0) and (StartIndex < SourceLen) then
  begin
    StartTag := '<' + Tag + '>';
    StartLen := Length(StartTag);

    if StartLen > 2 then
    begin
      StopTag := '</' + Tag + '>';
      StopLen := Length(StopTag);

      StartIndex := FastPosChecked(Source, StartTag, SourceLen, StartLen, StartIndex);
      if StartIndex > 0 then   // StartTag Found
      begin
        NestLevel := 1;
        Inc(StartIndex, StartLen);
        Index := StartIndex; // This is beginning of Content within Tag
        repeat
          StopIndex := FastPosChecked(Source, StopTag, SourceLen, StopLen, StartIndex);
          if StopIndex >= Index then
          begin
            Dec(NestLevel);
            repeat
              StartIndex := FastPosChecked(Source, StartTag, StopIndex, StartLen, StartIndex);
              if StartIndex > 0 then
              begin
                Inc(NestLevel);
                Inc(StartIndex, StartLen); 
              end;
            until StartIndex = 0;
            StartIndex := StopIndex + StopLen;
          end;
        until (NestLevel = 0) or (StopIndex = 0);

        if NestLevel = 0 then
        begin
          result := Copy(Source, Index, StopIndex - Index);
          Index := StopIndex + StopLen;
        end
        else
          Index := 0;
      end;
    end;
  end;
end;

procedure setXMLObject(Instance: TObject; p_sXML: AnsiString);
var
  CurrentTagIndex, OverAllIndex: Integer;
  CurrentTag, CurrentTagContent :string;
begin
  try
    CurrentTagIndex := 1;
    OverallIndex := 1;
    repeat
      CurrentTag := FastParseTag(p_sXML, '<' , '>', OverallIndex);
      CurrentTagContent := FastParseTagXML(p_sXML, CurrentTag, CurrentTagIndex);
      if (Length(CurrentTag) > 0) then
        SetPropAsString(Instance, GetPropInfo(Instance.ClassInfo, CurrentTag), CurrentTagContent);
  
      OverAllIndex := CurrentTagIndex;
    until (OverAllIndex<1) or (OverAllIndex > Length(p_sXML));
  except
    on EXMLException do
      raise;
    on e : Exception do
      raise EXMLException.Create('(' + e.Message + ')Error Processing XML - ' + CurrentTag);
  end;
end;

function isXMLPI(const TagStr : string):Boolean;
begin
  result := AnsiSameText('?xml', copy(TagStr,1,4));
end;

function getTagContent(const p_sXML, TargetTagName: String): string;
VAR
  CurrentTagIndex, OverAllIndex: Integer;
  CurrentTag, CurrentTagContent, DocType :string;
begin
  result := '';
  CurrentTagIndex := 1;
  OverallIndex := 1;
  DocType := FastParseTag(p_sXML, cs_DocTypeHeader, cs_DocTypeHeader_Close, OverallIndex);
  if length(DocType) = 0 then OverAllIndex := 1;
  repeat
    CurrentTag := FastParseTag(p_sXML, '<' , '>', OverallIndex);
    if not isXMLPI(CurrentTag) then
    begin
      CurrentTagContent := FastParseTagXML(p_sXML, CurrentTag, CurrentTagIndex);
      OverAllIndex := CurrentTagIndex;
    end;
  until (OverAllIndex<1) or (OverAllIndex > Length(p_sXML)) or SameText(CurrentTag, TargetTagName);

  if SameText(CurrentTag, TargetTagName) then
    result := CurrentTagContent;
end;



{:
  Generic File and Stream Routines 
}

procedure SaveStringToFile(const Str, FileName:string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate or fmShareDenyNone);
  try
    SaveStringToStream(Str, Stream);
  finally
    Stream.Free;
  end;
end;{ SaveStringToFile() }

procedure SaveStringToStream(const Str:string; Stream:TStream);
var
  Buffer : PChar;
  MemSize : Integer;
begin
  if not Assigned(Stream) then
    raise EXMLIOException.Create('Could not save to Stream: Stream is not assigned');
  MemSize := Length(Str);
  Buffer := AllocMem(MemSize+1);
  try
    StrPCopy(Buffer,Str);
    Stream.Seek(0,soFromBeginning);
    Stream.Write(Buffer^, MemSize);
  finally
    FreeMem(Buffer, MemSize);
  end;
end;{ SaveStringToStream() }

function LoadStringFromFile(const FileName:string): string;
var
  Stream : TFileStream;
begin
  Result := '';
  if FileExists(Filename) then
  begin
    Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyNone);
    try
      Result := LoadStringFromStream(Stream);
    finally
      Stream.Free;
    end;
  end;
end;{ LoadStringFromFile() }



function LoadStringFromStream(Stream:TStream): string;
var
  Buffer : PChar;
  MemSize : Integer;
begin
  if not Assigned(Stream) then
    raise EXMLIOException.Create('Could not load from Stream: Stream is not assigned');

  Result := '';
  MemSize := Stream.Size;

  Inc(MemSize);
  Buffer := AllocMem(MemSize);
  try
    Stream.Read(Buffer^, MemSize);
    Result := StrPas(Buffer);
  finally
    FreeMem(Buffer, MemSize);
  end;
end; { LoadStringFromStream() }

{
******************************** TXMLCollection ********************************
}
constructor TXMLCollection.Create(ItemClass: TXMLCollectionItemClass);
begin
  inherited Create(ItemClass);
end;

function TXMLCollection.Add: TXMLCollectionItem;
begin
  Result := inherited Add as TXMLCollectionItem;
end;

function TXMLCollection.FindInt(Value: Integer; Compare: 
        TCollectionIntegerCompare): TXMLCollectionItem;
var
  L, H, I, C: Integer;
begin
  result := nil;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    result := Items[I];
    C := -Compare(Value, Result);
    if C < 0 then
      L := I + 1
    else
    begin
      H := I - 1;
      if C = 0 then
        exit;
    end;
  
    result := nil;
  end;
end;

function TXMLCollection.FindStr(Value: String; Compare: 
        TCollectionStringCompare): TXMLCollectionItem;
var
  L, H, I, C: Integer;
begin
  result := nil;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    result := Items[I];
    C := -Compare(Value, Result);
    if C < 0 then
      L := I + 1
    else
    begin
      H := I - 1;
      if C = 0 then
        exit;
    end;
  
    result := nil;
  end;
end;

function TXMLCollection.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLCollection.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
  if result = '' then
    result := getItemTagName + '*'
  else
    result := getItemTagName + '*, ' + result;
end;

function TXMLCollection.GetElementText: string;
var
  Index: Integer;
  StringList: TStringList;
  CollectionItemName: string;
begin
  result := '';
  StringList := TStringList.Create;
  try
    StringList.Text := getPropertiesXML;
    CollectionItemName := GetItemTagName;
    for Index := 0 to Count-1 do
    begin
      StringList.Add('<' + CollectionItemName + '>' + Items[Index].ElementText + '</' + CollectionItemName + '>');
    end;
    result := StringList.Text;
  finally
    StringList.Free;
  end;
end;

function TXMLCollection.getItemTagName: string;
begin
  result := TXMLCollectionItemClass(ItemClass).getTagName;
end;

function TXMLCollection.getPropertiesXML: string;
begin
  result := ObjectToXMLElements(self)
end;

function TXMLCollection.getSubsetElementText(Start, Ct: Integer): string;
var
  StringList: TStringList;
  Index: Integer;
begin
  if (Start < 0) or (Start >= Count) then
    raise EXMLException.Create('Invalid Start (' + IntToStr(Start) + ')');
  
  if (Ct < 0) or ((Start + Ct) >= Count) then
    raise EXMLException.Create('Invalid Ct (' + IntToStr(Ct) + ')');
  
  result := '';
  StringList := TStringList.Create;
  try
    StringList.Text := ObjectToXMLElements(self);
    for Index := Start to Start+Ct-1 do
    begin
      StringList.Add(Items[Index].XML);
    end;
    result := StringList.Text;
  finally
    StringList.Free;
  end;
end;

class function TXMLCollection.getTagName: string;
begin
  result := Copy(ClassName, 2, Pos('Collection', ClassName) - 2) + '-list';
end;

function TXMLCollection.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;


function TXMLCollection.GetXMLCollectionItem(Index:Integer): TXMLCollectionItem;
begin
  Result := inherited Items[Index] as TXMLCollectionItem;
end;

procedure TXMLCollection.LoadFromFile(FileName:string);
begin
  XML := LoadStringFromFile(FileName);
end;

procedure TXMLCollection.SaveToFile(FileName:string);
begin
  SaveStringToFile(cs_XMLProlog+XML, FileName);
end;

procedure TXMLCollection.SetElementText(const p_sXML: string);
var
  CurrentTagIndex, OverAllIndex: Integer;
  CollectionItemName, CurrentTag, CurrentTagContent: string;
begin
  Clear;
  
  CollectionItemName := GetItemTagName;
  
  CurrentTagIndex := 1;
  OverallIndex := 1;
  repeat
    CurrentTag := FastParseTag(p_sXML, '<' , '>', OverallIndex);
    CurrentTagContent := FastParseTagXML(p_sXML,  CurrentTag, CurrentTagIndex);
  
    if (Length(CurrentTag) > 0) then
    begin
      if (CurrentTag = CollectionItemName) then
        Add.ElementText := CurrentTagContent
      else
        SetPropAsString(Self, GetPropInfo(ClassInfo, CurrentTag), CurrentTagContent);
    end;
  
    OverAllIndex := CurrentTagIndex;
  until (OverAllIndex<1) or (OverAllIndex > Length(p_sXML));
end;

procedure TXMLCollection.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

procedure TXMLCollection.SetXMLCollectionItem(Index:Integer; Value: 
        TXMLCollectionItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TXMLCollection.Sort(Compare: TListSortCompare);
  
  procedure CollectionQuickSort(L, R: Integer);
  var
    I, J: Integer;
    P: TXMLCollectionItem;
  begin
    repeat
      I := L;
      J := R;
      P := Items[(L + R) shr 1];
      repeat
  
        while (Compare(Items[I], P) < 0) and (i < self.Count-1) do
          Inc(I);
  
        while (Compare(Items[J], P) > 0) do
          Dec(J);
  
        if I <= J then
        begin
  
          if I <> J then
          begin
            Items[J].Index := I;
            Items[I+1].Index := J;
          end;
  
          Inc(I);
          Dec(J);
        end;
  
      until I > J;
      if L < J then
        CollectionQuickSort(L, J);
      L := I;
    until I >= R;
  end;
  
begin
  if Count > 0 then
    CollectionQuickSort(0, Count-1);
end;

{
****************************** TXMLCollectionItem ******************************
}
function TXMLCollectionItem.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLCollectionItem.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
end;

function TXMLCollectionItem.GetElementText: string;
begin
  result := ObjectToXMLElements(self);
end;

class function TXMLCollectionItem.getTagName: string;
begin
  result := Copy(ClassName, 2, Pos('Collection', ClassName) - 2);
end;

function TXMLCollectionItem.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

function TXMLCollectionItem.getXMLAsProperties: string;
begin
  result := '<' + getTagName + ' ' + ObjectToXMLProperties(self) + ' />'
end;

procedure TXMLCollectionItem.SetElementText(const p_sXML: string);
begin
  SetXMLObject(Self, p_sXML);
end;

procedure TXMLCollectionItem.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

{
********************************** TXMLObject **********************************
}
constructor TXMLObject.Create(p_sXML: String = '');
begin
  inherited Create;
  if p_sXML <> '' then
    XML := p_sXML;
end;

function TXMLObject.GetDTD: string;
begin
  result := cs_DocTypeHeader + getTagName + ' [' + #13#10 +
            cs_ElementHeader + getTagName + ' (' + getDTDSequence + ') >' + #13#10 +
            getDTDElements + ']>'
end;

function TXMLObject.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLObject.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
end;

function TXMLObject.GetElementText: string;
begin
  result := ObjectToXMLElements(self);
end;

class function TXMLObject.getTagName: string;
begin
  result := Copy(ClassName, 2, Length(ClassName) - 1);
end;

function TXMLObject.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

procedure TXMLObject.LoadFromFile(FileName:string);
begin
  XML := LoadStringFromFile(FileName);
end;

procedure TXMLObject.SaveToFile(FileName:string);
begin
  SaveStringToFile(XML, FileName);
end;

procedure TXMLObject.SetElementText(const p_sXML: string);
begin
  SetXMLObject(Self, p_sXML);
end;

procedure TXMLObject.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

{
***************************** TXMLInterfacedObject *****************************
}
constructor TXMLInterfacedObject.Create(p_sXML: String = '');
begin
  inherited Create;
  if p_sXML <> '' then
    XML := p_sXML;
end;

function TXMLInterfacedObject.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLInterfacedObject.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
end;

function TXMLInterfacedObject.GetElementText: string;
begin
  result := ObjectToXMLElements(self);
end;

class function TXMLInterfacedObject.getTagName: string;
begin
  result := Copy(ClassName, 2, Length(ClassName) - 1);
end;

function TXMLInterfacedObject.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

procedure TXMLInterfacedObject.LoadFromFile(FileName:string);
begin
  XML := LoadStringFromFile(FileName);
end;

procedure TXMLInterfacedObject.SaveToFile(FileName:string);
begin
  SaveStringToFile(XML, FileName);
end;

procedure TXMLInterfacedObject.SetElementText(const p_sXML: string);
begin
  SetXMLObject(Self, p_sXML);
end;

procedure TXMLInterfacedObject.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;

{
*********************************** TXMLList ***********************************
}
function TXMLList.getDTDElements: string;
begin
  result := getObjectDTDElements(self);
end;

function TXMLList.getDTDSequence: string;
begin
  result := getObjectDTDSequence(self);
  if result = '' then
    result := getItemTagName + '*'
  else
    result := getItemTagName + '*, ' + result;
end;

function TXMLList.GetElementText: string;
var
  Index: Integer;
  StringList: TStringList;
  ItemTagName: string;
begin
  result := '';
  ItemTagName := getItemTagName;
  StringList := TStringList.Create;
  try
    StringList.Text := ObjectToXMLElements(self);
    for Index := 0 to Count-1 do
    begin
      StringList.Add('<' + ItemTagName + '>' + Strings[Index] + '</' + ItemTagName + '>');
    end;
    result := StringList.Text;
  finally
    StringList.Free;
  end;
end;

class function TXMLList.getItemTagName: string;
begin
  result := cs_XMLLIST_itemTagName;
end;

class function TXMLList.getTagName: string;
begin
  { TMYLIST -> MYLIST }
  result := Copy(ClassName, 2, Length(ClassName) - 2);
end;

function TXMLList.GetXML: string;
begin
  result := '<' + getTagName + '>' + ElementText + '</' + getTagName + '>';
end;

procedure TXMLList.SetElementText(const p_sXML: string);
var
  CurrentTagIndex, OverAllIndex: Integer;
  ItemTagName, CurrentTag, CurrentTagContent: string;
begin
  Clear;
  
  ItemTagName := getItemTagName;
  
  CurrentTagIndex := 1;
  OverallIndex := 1;
  repeat
    CurrentTag := FastParseTag(p_sXML, '<' , '>', OverallIndex);
    CurrentTagContent := FastParseTagXML(p_sXML, CurrentTag, CurrentTagIndex);
  
    if (Length(CurrentTag) > 0) then
    begin
      if (CurrentTag = ItemTagName) then
        Add(CurrentTagContent)
      else
        SetPropAsString(Self, GetPropInfo(ClassInfo, CurrentTag), CurrentTagContent);
    end;
  
    OverAllIndex := CurrentTagIndex;
  until (OverAllIndex<1) or (OverAllIndex > Length(p_sXML));
end;

procedure TXMLList.SetXML(const p_sXML: string);
begin
  ElementText := GetTagContent(p_sXML, getTagName);
end;


end.
